/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _BaseUploader2 = require('./BaseUploader');

var _BaseUploader3 = _interopRequireDefault(_BaseUploader2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AxiosUploader = function (_BaseUploader) {
    _inherits(AxiosUploader, _BaseUploader);

    function AxiosUploader() {
        _classCallCheck(this, AxiosUploader);

        return _possibleConstructorReturn(this, (AxiosUploader.__proto__ || Object.getPrototypeOf(AxiosUploader)).apply(this, arguments));
    }

    _createClass(AxiosUploader, [{
        key: 'preInit',
        value: function preInit() {
            /**
             * @type {string}
             */
            this.method = 'PUT';

            /**
             * @type {File}
             */
            this.file = null;

            /**
             * @type {number}
             */
            this.minProgressUpdateIntervalMs = 500;

            /**
             * This is IIS max httpRuntime@maxRequestLength value which is 2147482624 Kb
             * @type {number}
             */
            this.bytesMaxPart = 2097151 * 1024;

            /**
             * @type {object}
             */
            this.headers = {};

            /**
             * @type {string}
             */
            this.fileFieldName = 'file';

            /**
             * @type {number}
             */
            this._lastReportTime = null;

            /**
             * @type {number}
             */
            this._bytesStart = 0;

            /**
             * @type {number|null}
             */
            this._bytesEnd = null;

            this.axios = null;

            _get(AxiosUploader.prototype.__proto__ || Object.getPrototypeOf(AxiosUploader.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'start',
        value: function start() {
            this._startInternal();
        }
    }, {
        key: 'stop',
        value: function stop() {
            // super.start();
        }
    }, {
        key: 'getUrl',
        value: function getUrl() {
            var params = this.getParams();
            params.uids = [this.file.getUid()];

            return this.constructor.addUrlParams(this._url, params);
        }
    }, {
        key: '_startInternal',
        value: function _startInternal() {
            var _extends2,
                _this2 = this;

            this.trigger(_BaseUploader3.default.EVENT_START);

            var config = {
                method: this.method,
                url: this._url,
                onUploadProgress: this._onUploadProgress.bind(this),
                headers: _extends({}, this.headers, (_extends2 = {}, _defineProperty(_extends2, 'Content-Disposition', 'attachment; filename="' + encodeURI(this.file.getName()) + '"'), _defineProperty(_extends2, 'Content-Type', 'multipart/form-data'), _extends2))
            };

            var bytesTotal = this.file.getBytesTotal();

            this._bytesStart = this.file.getBytesUploaded();
            this._bytesEnd = Math.min(this._bytesStart + this.bytesMaxPart, bytesTotal);

            config.data = new FormData();
            config.data.append(this.fileFieldName, this.file.getNative());

            if (this.axios instanceof Promise) {
                Promise.resolve(this.axios).then(function (axios) {
                    axios.request(config).catch(function () {
                        _this2.trigger(_BaseUploader3.default.ERROR);
                    });
                });
            } else {
                this.axios.request(config).catch(function () {
                    _this2.trigger(_BaseUploader3.default.ERROR);
                });
            }
        }
    }, {
        key: '_onUploadProgress',
        value: function _onUploadProgress(progressEvent) {
            var iNow = new Date().getTime();
            if (this._lastReportTime && iNow - this._lastReportTime < this.minProgressUpdateIntervalMs) {
                return;
            }
            this._lastReportTime = iNow;

            if (progressEvent.total === progressEvent.loaded) {
                this.trigger(_BaseUploader3.default.EVENT_END);
            } else {
                this.trigger(_BaseUploader3.default.EVENT_PROGRESS, [progressEvent.loaded]);
            }
        }
    }], [{
        key: 'isProgressSupport',
        value: function isProgressSupport() {
            return true;
        }
    }]);

    return AxiosUploader;
}(_BaseUploader3.default);

exports.default = AxiosUploader;